package com.xzzn.ems.mapper;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import com.xzzn.ems.domain.EmsPcsData;
import com.xzzn.ems.domain.vo.ElectricIndexList;
import com.xzzn.ems.domain.vo.PcsDetailInfoVo;
import com.xzzn.ems.domain.vo.SiteMonitorDataVo;
import com.xzzn.ems.domain.vo.SiteMonitorRuningHeadInfoVo;

/**
 * PCS数据Mapper接口
 *
 * @author xzzn
 * @date 2025-06-17
 */
public interface EmsPcsDataMapper
{
    /**
     * 查询PCS数据
     *
     * @param id PCS数据主键
     * @return PCS数据
     */
    public EmsPcsData selectEmsPcsDataById(Long id);

    /**
     * 查询PCS数据列表
     *
     * @param emsPcsData PCS数据
     * @return PCS数据集合
     */
    public List<EmsPcsData> selectEmsPcsDataList(EmsPcsData emsPcsData);

    /**
     * 新增PCS数据
     *
     * @param emsPcsData PCS数据
     * @return 结果
     */
    public int insertEmsPcsData(EmsPcsData emsPcsData);

    /**
     * 修改PCS数据
     *
     * @param emsPcsData PCS数据
     * @return 结果
     */
    public int updateEmsPcsData(EmsPcsData emsPcsData);

    /**
     * 删除PCS数据
     *
     * @param id PCS数据主键
     * @return 结果
     */
    public int deleteEmsPcsDataById(Long id);

    /**
     * 批量删除PCS数据
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteEmsPcsDataByIds(Long[] ids);

    /**
     * 根据site_id获取日期对应的充放电数据
     * @param siteId
     * @return
     */
    public List<SiteMonitorDataVo> getPcsDataBySiteId(Long siteId);

    /**
     * 根据站点获取电网实时功率=sum（总交流有功电率）
     * @param siteId
     * @return
     */
    public BigDecimal getGridNrtPower(Long siteId);

    /**
     * 根据站点获取设备监控的实时运行头信息
     * @param siteId
     * @return
     */
    public SiteMonitorRuningHeadInfoVo getSiteRunningHeadInfo(Long siteId);

    /**
     * 获取每月的充电量和放电量
     * @return
     */
    public List<ElectricIndexList> getElectDataList();

    /**
     * 根据site_id获取所有pcs最新update的数据
     * @param siteId
     * @return
     */
    public List<PcsDetailInfoVo> getPcsDetailInfoBySiteId(Long siteId);

    /**
     * 获取总充+总放
     * @return
     */
    public Map<String, BigDecimal> getPcsTotalChargeData();
}
